C***********************************************************************
C*                                                                     *
C*                     SKYCONES (version 1.1)                          *
C*                                                                     *
C*  This program evaluates the gamma-ray or neutron skyshine dose      *
C*  from a bare point source collimated into a vertical annular cone   *
C*  between two polar angles THETA1 and THETA2.  Axial (azimuthal)     *
C*  symmetry is assumed for the source.  A sequence of annular cones   *
C*  can be analyzed and the skyshine from all the annulaar cones can   *
C*  be summed to yield the skyshine dose from a source whose intens-   *
C*  ity varies with polar angle.                                       *
C*                                                                     *
C*  The calculation is based on the integral line-beam method using    *
C*  an improved 3-parameter approximation for the line-beam response   *
C*  function. An infinite air medium is assumed, i.e., the ground is   *
C*  approximated by a half-space of air.                               *
C*                                                                     *
C*  The skyshine doses are for an infinite air medium, i.e., the       *
C*  ground is approximated by a continuation of the the air medium.    *
C*  This has little effect for gamma-photon sources.  However, for     *
C*  neutron sources, ground correction factors (GCF) may be applied    *
C*  to correct the infinite-air doses for soil beneath the air-ground  *
c*  interface.                                                         *
C*                                                                     *
C*  Source energy range must be as follows:                            *
C*               photons:   0.02 <= E <= 100 MeV                       *
C*               neutrons:  0.01 <= E <= 14  MeV                       *
C*  If energies outside the range are input, the energies are set to   *
C*  the closest energy limit.                                          *
C*                                                                     *
C*  References for the calculational methods are:                      *
C*  (1) J.K. Shultis, R.E. Faw and M.S. Bassett, "The Integral Line-   *
C*      Beam Method for Gamma Skyshine Analysis," Nucl. Sci. Engg.,    *
C*      107, 228-245 (1991).                                           *
C*  (2) J.K. Shultis, and R.E. Faw, "Extensions to the Integral Line-  *
C*      Beam Method for Gamma-Ray Skyshine Analyses," SAND-94-2019,    *
C*      Sandia National Lab., Albuquerque, NM, 1994.                   *
C*  (3) A.A. Gui, J.K. Shultis, and R.E. Faw, "Neutron Skyshine Calc-  *
C*      ulations with the Integral Line-Beam Method," Nucl. Sci. Engg. *
C*      127, 230-237 (1997).                                           *
C*                                                                     *
C*  Data Files Needed for LBRF approximations and GCFS                 *
C*     Photon data:   GAMLBRF.DAT                                      *
C*     Neutron data:  NEUTLBRF.DAT and NEUTGCF.DAT                     *
C*                                                                     *
C*  Adapted from SKYDOSE, SKYNEUT, and SKYCOS by J.K. Shultis 6/27/99  *
C***********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*32 OUTFIL,INNAM,FD
      CHARACTER*2 RADTYP,dum
      CHARACTER*23 Dunit
      CHARACTER*7 SRC
      CHARACTER*80 title
      COMMON/GAUS01/X(32),W(32)
      COMMON/ECOM/E(100),P(100),NE,title,Dunit,SRC
      COMMON/DSUMM/SDD(100),SDOSE(100),SDOSE2(100)
      FD = '(1X,A,$)'
C
C****** PRINT OPENING SCREEN
      WRITE(*,20)
      WRITE(*,21)
C
C****** ASK USER FOR INPUT FILE AND OPEN OUTPUT FILE
C** Use only input file (no keyboard input)
      PRINT *
      WRITE(*,FD) 'Enter name of input file for skyshine problem data
     1 ===> '
      READ(*,1) INNAM
      OPEN(18,FILE=INNAM)
      READ(18,1,END=99) OUTFIL

C** READ RADIATION TYPE: (Photon INDX=0; Neutron INDX=1,2,3)
      READ(18,1,END=99) RADTYP
      IF (RADTYP(1:1).EQ.'P'.OR.RADTYP(1:1).EQ.'p') THEN
         INDX=0
         Dunit='air-Gy and roentgen    '
         SRC='Photon '
         CALL XBINI
      ELSEIF (RADTYP(1:1).EQ.'N'.OR.RADTYP(1:1).EQ.'n') THEN
         INDX=2
         Dunit='ADE - ICRU Sphere (PAR)'
         SRC='Neutron'
         CALL NBINI
         IF(RADTYP(2:2).EQ.'G'.OR.RADTYP(2:2).EQ.'g') THEN
            CALL GCFINI
            INDX=-2
         ENDIF
      ELSEIF (RADTYP.EQ.'E'.OR.RADTYP.EQ.'e') THEN
         INDX=1
         Dunit='EDE - ICRP Phantom (AP)'
         SRC='Neutron'
         CALL NBINI
         IF(RADTYP(2:2).EQ.'G'.OR.RADTYP(2:2).EQ.'g') THEN
            CALL GCFINI
            INDX=-1
         ENDIF
      ELSEIF (RADTYP.EQ.'I'.OR.RADTYP.EQ.'i') THEN
         INDX=3
         Dunit='DE - ICRU Sphere (ISO)'
         SRC='Neutron'
         CALL NBINI
         IF(RADTYP(2:2).EQ.'G'.OR.RADTYP(2:2).EQ.'g') THEN
            CALL GCFINI
            INDX=-3
         ENDIF
      ELSE
         PRINT*,'****** FATAL ERROR: RADTYP must be P or N (or E,I)'
         STOP
      ENDIF

C
C
C** Initialize cumulative sum
      ISTRT=0
C-- Initialize the sum vectors for cumulative dose
      IADD=1
      DO 81 I=1,100
         SDOSE(I)=0.0D0
         SDOSE2(I)=0.0D0
81    CONTINUE
C
C
C*** GET INPUT DATA COMMON TO ALL GEOMETRIES
98       READ(18,1,END=99) dum
         READ(18,1,END=99) title
         READ(18,*,END=99) RHO,NPTS,NGAU,NE,IAD
         IADD=IAD*IADD
         IPROB=1
         DO 82 I=1,NE
            READ(18,*,END=99)  E(I),P(I)
82       CONTINUE
C
C
C*** SET CONSTANTS, SET UP QUADRATURE, OPEN OUTPUT FILE
      PI = DACOS(-1.D0)
      CALL READGS(NGAU,X,W)
C
      IF (ISTRT.EQ.0) THEN
         OPEN(12,FILE=OUTFIL,STATUS='UNKNOWN')
         ISTRT=1
      ENDIF
C
      IF (NPTS.LT.0) THEN
        NPTS=NPTS-1
      ELSE
        NPTS = NPTS + 1
      ENDIF
C
C
C********** CALL THE GEOMETRY INPUT ROUTINE
       CALL SILOIO(INDX,RHO,NGAU,NPTS)
C -- do another case
        GOTO 98
C
C********* Print out final dose sum (if IADD=1)
99       IF(IADD.EQ.1) THEN
            IF (INDX.EQ.0) THEN
              WRITE(12,35)
              WRITE(6,35)
            ELSE
               WRITE(12,37)
               WRITE(6,37)
            ENDIF
            DO 97 I=1,IABS(NPTS)
              WRITE(12,30)SDD(I),100.D0*SDD(I)*RHO,SDOSE(I),SDOSE2(I)
              WRITE(6,30)SDD(I),100.D0*SDD(I)*RHO,SDOSE(I),SDOSE2(I)
97          CONTINUE
            WRITE(12,36)
            WRITE(6,36)
         ENDIF
         STOP
C
C
1     FORMAT(A)
118   FORMAT (1X)
20    FORMAT(22(/),22X,23('*'),/22X,'***** SKYCONES 1.1 *****'/
     1       22X,23('*')/)
21    FORMAT(' This program evaluates skyshine doses from point photon o
     1r neutron   '/' sources.  The source may be collimated into an ann
     2ular cone opening  '/' vertically upward.  The method is based on
     3the integral line-   '/'beam response function techniques using a
     43-parameter approximation'/'for the LBRF.'//)
30    FORMAT(F12.1,F12.2,2X,2(1PE13.4))
35    FORMAT(///10X,'SUM OF ABOVE SKYSHINE DOSES',/2X,54('-'),/
     1  6X,'S-D(m)',6X,'g/cm^2',5X,'Gy/photon',5X,'R/photon',
     2  /2x,54('-'))
37    FORMAT(///10X,'SUM OF ABOVE SKYSHINE DOSES',/2X,54('-'),/
     1  8X,'S-D',9X,'S-D',7X,'Neutron',5X,'Sec. Gamma',/
     2  8X,'(m)',6X,'(g/cm^2)',4X,'(Sv/neut)',4X,'(Sv/neut)'/
     3  2X,(54('-')))
36    FORMAT(2X,54('-'))
C
      END


      SUBROUTINE SILOIO(INDX,RHO,NGAU,N)
C**********************************************************************
C  Purpose:
C   This subroutine (1) reads geometric data for the open silo
c   (conical) source, (2) calls for the skyshine dose, and
c   (3) outputs the results.
C
C  Input variables:
C  INDX   = 0 for gamma rays; = 1,2,3 for neutrons (-1,-2,-3 for GCF)
C   RHO   =  air density (g/cm^3)
C  NGAU   =  order of Gaussian quadrature (4, 8, 16 or 32)
C     N   =  no. of intermeduate source-detector distances
C            (equi-x for N>0; equi-logx if N<0)
C
C  Silo geometry variables (read from input file by this subroutine):
C     DELH = source elevation less detector elevation
C       XD = horizontal dist. from source to detector farthest away
C   THETA1 = minimum polar angle (deg.) (0 <= theta1 < theta2 <= 90)
C   THETA2 = maximum polar angle (deg.) (0 <= theta1 < theta2 <= 90)
C
c**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*9 FD
      CHARACTER*80 title
      CHARACTER*23 Dunit
      CHARACTER*7 SRC
      COMMON/ECOM/E(100),P(100),NE,title,Dunit,SRC
      COMMON/DSUMM/SDD(100),SDOSE(100),SDOSE2(100)
      FD = '(/1X,A,$)'
C
C*** READ GEOMETRY DATA
C     READ(18,*,END=41,ERR=41) HS,HD,R,XD
      READ(18,*,END=41,ERR=41) DELH, XD, theta1, theta2
      IF(theta2.LE.theta1) GOTO 41
C
C
C** CALCULATE SOURCE COLLIMATION SOLID ANGLE
42    PI=DACOS(-1.D0)
      W1=DCOS(PI*theta1/180.D0)
      W2=DCOS(PI*theta2/180.D0)
      OMEGA=2.D0*PI*(W1-W2)
C
C*** PRINT GEOMETRY DATA INTO OUTPUT FILE and check input data
      WRITE(12,15) title,SRC,RHO,DELH,THETA1,THETA2,OMEGA,XD,NGAU
      WRITE(6,15) title,SRC,RHO,DELH,THETA1,THETA2,OMEGA,XD,NGAU
      IF (INDX.LT.0) THEN
         WRITE(12,17)
         WRITE(6,17)
      ENDIF
      WRITE(12,16) (E(K),P(K),K=1,NE)
      WRITE(6,16) (E(K),P(K),K=1,NE)
      CALL ERRCHK(INDX,XD,theta1,theta2)
      IF(INDX.EQ.0) THEN
         WRITE(12,25) Dunit
         WRITE(6,25) Dunit
      ELSE
         WRITE(12,27) Dunit
         WRITE(6,27) Dunit
      ENDIF
C
C
C
C********* BEGIN CALCULATIONS FOR ALL DISTANCES
      NX=IABS(N)
      DO 888 I=1,NX
         IF (N.GT.0) THEN
            X = (XD/REAL(N))*I
         ELSE
            X = DEXP(DLOG(5.d0) + I*DLOG(XD/5.d0)/REAL(NX))
         ENDIF
         XI=DATAN((DELH)/X)
         DD=DSQRT((DELH)**2+X*X)
C
C****** Sum over each energy bin: separate photon and neutrons sources
c--- photon source
       IF (INDX.EQ.0) THEN
           SUME=0.0D0
           DO 10, J=1,NE
              ENRGY = E(J)
              CALL SILO(INDX,ENRGY,RHO,NGAU,W1,W2,DD,XI,DOSE)
              SUME = SUME + DOSE*P(J)
10         CONTINUE
C===> Divide dose (air-rad) by 100 to yield (air-Gy)
           DOSE = SUME/100.d0
c--- neutron source
       ELSE
           SUMN=0.0
           SUMG=0.0
           NG = INDX+3
           IF (INDX.LT.0) NG=INDX-3
           DO 11 J=1,NE
              ENGY=E(J)
              CALL SILO(INDX,ENGY,RHO,NGAU,W1,W2,DD,XI,DOSEN)
              SUMN=SUMN+DOSEN*P(J)
              CALL SILO(NG,ENGY,RHO,NGAU,W1,W2,DD,XI,DOSEG)
              SUMG=SUMG+DOSEG*P(J)
11         CONTINUE
C===>  NOTE: divide neutron dose by factor of 2 to correct to US stnd
         DOSEN=SUMN/2.0D0
         DOSEG=SUMG
       ENDIF
C
C*** PRINT OUT RESULTS
       IF (INDX.EQ.0) THEN
C===> Multiply air-Gy by 100 for air-rad and by 1.145 for R
         DOSE2 = 114.5d0*DOSE
       ELSE
         DOSE = DOSEN
         DOSE2 = DOSEG
       ENDIF
       WRITE(12,20)DD,100.D0*DD*RHO,DOSE,DOSE2
       WRITE(6,20) DD,100.D0*DD*RHO,DOSE,DOSE2
C**  Add to cumulative sum
         SDD(I)=DD
         SDOSE(I)=SDOSE(I)+DOSE
         SDOSE2(I)=SDOSE2(I)+DOSE2

 888  CONTINUE
      WRITE(12,26)
      WRITE(6,26)
      RETURN
C
C** Error in input data file
41    PRINT*,'**** Error in data file: Program stopped'
      STOP
C
C
15    FORMAT(///A80,/'****** SKYCONES 1.1: Annular Conical Geometry',
     1   ' with ',A7,' Source',/
     2       '  Air density (g/cm^3)       =',F10.6,/
     3       '  Dist. src above detect (m) =',F10.2,/
     4       '  Source angle 1 (degrees)   =',F10.2,/
     5       '  Source angle 2 (degrees)   =',F10.2,/
     6       '  Source collimation (sr)    =',F10.4,/
     7       '  Source-detector dist. (m)  =',F10.1,/
     8       '  Gauss quadrature order     =',I10,/
     9       '  3-parm. approx. LBRF used ')
17    FORMAT('  Ground correction factors applied to neutron dose')
16    FORMAT(/'  Energy Spectrum: Emid (MeV)   Frequency:'
     1       /(15X,F12.4,G15.4))
20    FORMAT(F12.1,F12.2,2X,2(1PE13.4))
25    FORMAT(/10X,'SKYSHINE DOSE: ',A23,/2X,54('-'),/
     1  6X,'S-D(m)',6X,'g/cm^2',5X,'Gy/photon',5X,'R/photon',
     2  /2x,54('-'))
27    FORMAT(/10X,'SKYSHINE DOSE: ',A23,/2X,54('-'),/
     1  8X,'S-D',9X,'S-D',7X,'Neutron',5X,'Sec. Gamma',/
     2  8X,'(m)',6X,'(g/cm^2)',4X,'(Sv/neut)',4X,'(Sv/neut)'/
     3  2X,(54('-')))
26    FORMAT(2X,54('-'))
      END


       SUBROUTINE SILO(INDX,E,RHO,NGAU,W1,W2,ZZ,XI,DOSE)
C********************************************************************
C  Purpose:
C   This subroutine calculates the skyshine dose for a source in
C   an open cylindrical silo on the silo's axis whose concical beam
C   is annularly collimated between two polar angles.
C
C  Input:
C  INDX = 0 for photons; =1,2,3 for neutrons(-1,-2,-3 for GCF)
C                        =4,5,6 for sec. photons (-4,,-5,-6 for GCF)
C     E = particle energy (MeV)
C   RHO =  air density (g/cm^3)
C  NGAU = Gaussian quadrature order
C    W1 = cosine of the minimum polar angle (silo collimation)
C    W2 = cosine of the maximun polar angle (silo collimation)
C    ZZ = source-to-detector distance (m)
C    XI = angle between the horizontal axis and source-detector axis
C
C  Output:
C    DOSE = skyshine dose (rad/photon or DE/neutron)
C
C   NEEDED SUBROUTINES:
C      XBEAM = Approximate LBRF for photons
C      NBEAM = Approximate LBRF for neutrons
C********************************************************************
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
       COMMON/GAUS01/X(32),W(32)
C
      PI = DACOS(-1.D0)
      NDX = ABS(INDX)
      GCF = 1.d0
C*** INTEGRATE LBRF OVER ALL AZIMUTHAL ANGLES 0 TO PI
      TOTSUM = 0.0D0
      DO 100 L = 1,NGAU
         WPSI = 0.5*PI*W(L)
         PSI = 0.5*PI*(X(L) + 1.D0)
C
C*** INTEGRATE OVER POLAR ANGLES
         TOTANS=0.0D0
         DO 99 K = 1,NGAU
           WTHE = 0.5*(W1-W2)*W(K)
           WW = 0.5*(W1-W2)*X(K) + 0.5*(W1+W2)
           IF(WW.GT.1.D0)GOTO 110
           DWW = (1.D0 - WW*WW)**0.5
           Theta = DACOS(WW)*180.d0/PI
C*** CALCULATE BEAM EMISSION ANGLE PHI
           DPHI = DCOS(PSI)*DWW*DCOS(XI)+WW*DSIN(XI)
           PHI = DACOS(DPHI)*180.d0/PI
C*** CALCULATE SKYSHINE DOSE FOR GIVEN PARTICLE TYPE
           IF (NDX.EQ.0) THEN
              CALL XBEAM(E,PHI,ZZ,RHO,RESP)
           ELSE
              CALL NBEAM(NDX,E,PHI,ZZ,RHO,RESP)
C - Calculate ground correction factor if requested
              IF (INDX.LT.0) THEN
                 IF (NDX.GT.3) THEN
                    CALL NGCF(NDX-3,E,theta,ZZ,DUM,GCF)
                 ELSE
                    CALL NGCF(NDX,E,theta,ZZ,GCF,DUM)
                 ENDIF
              ENDIF
              RESP=RESP*GCF
           ENDIF
           TOTANS = TOTANS + WTHE*RESP
99      CONTINUE
C
        TOTSUM = TOTSUM + WPSI*TOTANS
100   CONTINUE
C
      DOSE=0.5*TOTSUM/PI
110   RETURN
      END

      SUBROUTINE ERRCHK(INDX,XD,theta1,theta2)
C*************************************************************************
C  Check energy and space ranges.
C*************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*110 dummy
      COMMON/ECOM/E(100),P(100),NE,dummy

      IF (INDX.EQ.0) THEN
         EMIN = 0.02D0
         EMAX = 100.0D0
      ELSE
         EMIN = 0.01D0
         EMAX = 14.0D0
      ENDIF

C** Check for energy range and reset if outside allowed range
      EEBIG=0.0D0
      DO 10 K=1,NE
         IF(E(K).LT.EMIN) THEN
            WRITE(12,11) E(K),EMIN
            WRITE(6,11) E(K),EMIN
            E(K)=EMIN
         ELSEIF(E(K).GT.EMAX) THEN
            WRITE(12,12) E(K),EMAX
            WRITE(6,12) E(K),EMAX
            E(K)=EMAX
         ENDIF
         IF(E(K).GT.EEBIG) EEBIG=E(K)
10    CONTINUE

C*** Check for maximum ranges
      IF (INDX.EQ.0) THEN
         IF (EEBIG.LE.10.D0.AND.XD.GT.3000.D0) THEN
            WRITE(12,15)
            WRITE(6,15)
         ENDIF
         IF (EEBIG.GT.10.D0.AND.XD.GT.1000.D0) THEN
            WRITE(12,16)
            WRITE(6,16)
         ENDIF
      ELSE
         IF (XD.GT.2500.D0) THEN
            WRITE(12,17)
            WRITE(6,17)
         ENDIF
      ENDIF

C**** Check angular range
      IF (theta1.LT.0.D0.OR.theta1.GT.90.D0) THEN
          WRITE(6,20) theta1
          WRITE(12,20) theta1
          STOP
      ENDIF
      IF (theta2.LT.0.D0.OR.theta1.GT.90.D0) THEN
          WRITE(6,21) theta2
          WRITE(12,21) theta2
          STOP
      ENDIF
      IF (theta1.GE.theta2) THEN
          WRITE(6,22)
          WRITE(12,22)
          STOP
      ENDIF
c
      RETURN
c
11    FORMAT('**** WARNING: Source energy ',
     1 F8.6,' MeV below minimum. Set to ',F8.6, ' MeV')
12    FORMAT('**** WARNING: Source energy ',
     1 F5.1,' MeV above maximum. Set to ',F5.1, ' MeV')
15    FORMAT('**** WARNING: S-D dist. > LBRF 3000m range (Emax<10 MeV)')
16    FORMAT('**** WARNING: S-D dist. > LBRF 1000m range (Emax>10 MeV)')
17    FORMAT('**** WARNING: S-D dist. > neutron LBRF 2500m range')
20    FORMAT('**** FATAL ERROR: THETA1 = ',F6.2,
     1       '  Must be between 0 and 90 deg.')
21    FORMAT('**** FATAL ERROR: THETA2 = ',F6.2,
     1       '  Must be between 0 and 90 deg.')
22    FORMAT('**** FATAL ERROR: THETA2 <= THETA1')

      END


      BLOCK DATA EAGRID
C*************************************************************************
C  Initialize data for XBEAM, NBEAM, and NGCF
C*************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /GAMVEC/ EEX(2,15),PPSIX(2,20)
      COMMON /NVEC/ EEN(14),PPSIN(18)
      COMMON /GCFDAT/ EEg(11),Thetag(8),GCFN(3,11,8,4)

C***** GAMMA RAYS:
C***** Discrete angles for calculation of approximate lbrf
C      Energy < 10.0 MeV
        DATA (PPSIX(1,I),I=1,20)
     *       /0.5,1.5,2.5,4.0,6.0,8.5,12.5,17.5,25.0,35.0,45.0,
     *        55.0,65.0,75.0,85.0,95.0,110.,130.,150.,170./
C      Energy > 10.0 MeV
        DATA (PPSIX(2,I),I=1,17)
     *       /2.0,3.5,5.0,7.5,10.0,15.0,20.0,30.0,45.0,60.0,
     *        75.0,85.0,95.0,105.0,120.0,135.0,150.0/
C***** Discrete energies for calculation of approximate lbrf
C      Energy < 10.0 MeV
        DATA (EEX(1,I),I=1,15)
     *       /0.02,0.03,0.04,0.06,0.08,0.1,0.2,0.4,0.7,1.0,
     *        2.0,4.0,7.0,10.,15./
C      Energy > 10.0 MeV
        DATA (EEX(2,I),I=1,5)/10.0,20.0,40.0,70.0,100.0/
C
C***** NEUTRONS:
C***** Discrete angles for calculation of approximate Neut. and Gamma lbrf
        DATA (PPSIN(I),I=1,18)
     *       /1.0,2.0,4.0,6.0,8.0,12.0,18.0,25.0,35.0,45.0,
     *        55.0,65.0,75.0,90.0,110.0,130.0,150.0,170.0/
C***** Discrete energies for calculation of approximate Neut. and Gamma lbrf
        DATA (EEN(I),I=1,14)
     *       /0.01,0.05,0.1,0.5,1.0,1.5,2.0,2.4,3.0,5.0,
     *        7.0,9.0,11.0,14.0/
C***** Ground correction factors: Neutron Source
C**    Discrete angles for calculation of GCF
       DATA (Thetag(I),I=1,8)
     1      /1.d0, 25.d0,35.d0,45.d0,55.d0,65.d0,75.d0,85.d0/
C**    Discrete energies for calculation of GCF
       DATA (EEg(I),I=1,11)
     1      /0.1d0,0.5d0,1.5d0,3.0d0,3.5d0,4.0d0,4.5d0,5.d0,
     2       7.d0,11.d0,14.d0/
       END


      SUBROUTINE XBINI
C**********************************************************************
C  Reads data file containing coefficients for the three-parameter
C  approximation to the LBRF for gamma photons.
C  Call once before using subprogram XBEAM.
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*15 EHDR,HDR
      COMMON /GAMDAT/ G(2,15,20,3),ANGL(2,20)
C
C ****** Read in the fit parameters for the approximate LBRF
      OPEN(21,FILE='GAMLBRF.DAT')
C** Read low-E 3-parm data
      READ(21,10) HDR
      DO 20 I=1,15
        READ(21,10) EHDR
        READ(21,*)(ANGL(1,J),G(1,I,J,1),G(1,I,J,2),G(1,I,J,3),J=1,20)
20    CONTINUE
C** Read high-E 3-parm data
      READ(21,10) HDR
      DO 30 I=1,5
        READ(21,10) EHDR
        READ(21,*)(ANGL(2,J),G(2,I,J,1),G(2,I,J,2),G(2,I,J,3),J=1,17)
30    CONTINUE

10    FORMAT(A)
      CLOSE(21)
      RETURN
      END


      SUBROUTINE NBINI
C**********************************************************************
C  Reads data file containing coefficients for the three-parameter
C  approximation to the LBRF for neutrons and secondary gammas.
C  Call once before using subprogram NBEAM.
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*15 EHDR,HDR
      COMMON /NDAT/ GN(6,14,18,3),ANGLN(18)
C
      OPEN(21,FILE='neutlbrf.dat')
C
C *****  Read in the fit parameters for the approximate LBRF
C** Read Neutron and Sec.-Gamma Parameters (AP)
      READ(21,10) HDR
      DO 20 I=1,14
        READ(21,10) EHDR
        READ(21,*)(ANGLN(J),GN(1,I,J,1),GN(1,I,J,2),GN(1,I,J,3),J=1,18)
20    CONTINUE
      READ(21,10) HDR
      DO 30 I=1,14
        READ(21,10) EHDR
        READ(21,*)(ANGLN(J),GN(4,I,J,1),GN(4,I,J,2),GN(4,I,J,3),J=1,18)
30    CONTINUE

C** Read Neutron and Gamma Parameters (PAR)
      READ(21,10) HDR
      DO 40 I=1,14
        READ(21,10) EHDR
        READ(21,*)(ANGLN(J),GN(2,I,J,1),GN(2,I,J,2),GN(2,I,J,3),J=1,18)
40    CONTINUE
      READ(21,10) HDR
      DO 50 I=1,14
        READ(21,10) EHDR
        READ(21,*)(ANGLN(J),GN(5,I,J,1),GN(5,I,J,2),GN(5,I,J,3),J=1,18)
50    CONTINUE

C** Read Neutron and Gamma Parameters (ISO)
      READ(21,10) HDR
      DO 60 I=1,14
        READ(21,10) EHDR
        READ(21,*)(ANGLN(J),GN(3,I,J,1),GN(3,I,J,2),GN(3,I,J,3),J=1,18)
60    CONTINUE
      READ(21,10) HDR
      DO 70 I=1,14
        READ(21,10) EHDR
        READ(21,*)(ANGLN(J),GN(6,I,J,1),GN(6,I,J,2),GN(6,I,J,3),J=1,18)
70    CONTINUE

      CLOSE(21)
10    FORMAT(A)
      RETURN
      END


      SUBROUTINE GCFINI
C**********************************************************************
C  Reads data file containing coefficients for the approximations
C  of the ground correction factors (GCF) for neutron conical beams.
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*15 EHDR
      COMMON /GCFDAT/ EEg(11),Thetag(8),GCFN(3,11,8,4)
C
C ****** Read in the fit parameters for the approximate GCF
C **     Store in NGCF(set,E,angle,parm)
      OPEN(21,FILE='neutgcf.dat')
      DO 20 I=1,11
        READ(21,10) EHDR
        READ(21,*)(Thetag(J),
     1       GCFN(1,I,J,1),GCFN(1,I,J,2),GCFN(1,I,J,3),GCFN(1,I,J,4),
     2       GCFN(2,I,J,1),GCFN(2,I,J,2),GCFN(2,I,J,3),GCFN(2,I,J,4),
     3       GCFN(3,I,J,1),GCFN(3,I,J,2),GCFN(3,I,J,3),J=1,8)
c       print *,Thetag(1),
c    1       GCFN(1,I,1,1),GCFN(1,I,1,2),GCFN(1,I,1,3),GCFN(1,I,1,4),
c    2       GCFN(2,I,1,1),GCFN(2,I,1,2),GCFN(2,I,1,3),GCFN(2,I,1,4),
c    3       GCFN(3,I,1,1),GCFN(3,I,1,2),GCFN(3,I,1,3)
c     PAUSE
20    CONTINUE
      CLOSE(21)

10    FORMAT(A)
      RETURN
      END




      SUBROUTINE NGCF(INDX,E,PSI,ZZ,CFN,CFP)
C***************************************************************************
C  Purpose:
C    This subroutine calculates the ground correction factor for conical
C    neutron beams.
C
C  Input:
C   INDX = 1 (EDE: AP-ICRP phantom); =2 (ADE: PAR-ICPR sphere)
C        = 3 (DE: ISO-ICRP sphere)
C      E = Source energy (MeV),
C    PSI = conical beam polar half-angle (deg.)
C     ZZ = source-to-detector distance (m)
C
C  Output:
C     CFN = Ground correction factor for neutron skyshine
C     CFP = Ground correction factor for secondary photons
C           NOTE: in this version of SKYCONES, CFP is set to 1
C
C  Variables:
C        FK     = flux-to-dose conversion factor
C        IE     = energy group index in which energy E resides
C        IEADJ  = adjacent energy index
C        JP     = angular group index in which a beam angle resides
C        JPADJ  = adjacent angle index
C
C  Needed Routines:  GCFINI to read GCF approximation coefficients
C                    EAGRID Block Data to set up E and ANG grid
C
C**************************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        DIMENSION  FF(20),FIE(20),FIEADJ(20)
        DIMENSION  PF(20),PIE(20),PIEADJ(20)
        COMMON /GCFDAT/ EEg(11),Theta(8),GCFN(3,11,8,4)

        NE = 11
        NANG = 8

c******* Set limits on energy within GCF data base (i.e. no extrapolation)
        EE=E
        IF(E.LE.0.1d0) EE=0.1d0
        IF(E.GE.14.d0) EE=14.d0

c******* Set limits on distance for neutron GCF
        XN = ZZ
        IF (EE.LT.0.25d0.AND.ZZ.GT.700.d0) XN=700.d0
        IF (EE.GE.0.25d0.AND.ZZ.GT.1100.d0)  XN=1100.d0

c  Calculate which GCF data set to use
       IF (INDX.EQ.2) THEN
         NDX = 2
       ELSE
         NDX = 1
       ENDIF

        RHOAIR = 0.0012D0
        RFACT = 1.d0
c        RFACT = RHOACT/RHOAIR

C** Find energy indices IE and IEADJ bracketing energy E

        IF (EE.LE.EEg(1)) THEN
            IE = 1
        ELSEIF (EE.GE.EEg(NE)) THEN
            IE = NE
        ELSE
            DO 10 I=2,NE
              IF(EE.GE.EEg(I-1).AND.EE.LT.EEg(I)) THEN
                 IE = I-1
                 GOTO 11
              ENDIF
  10        CONTINUE
        ENDIF
  11    EIE = EEg(IE)
        IEADJ = IE+1
        IF(IE.EQ.NE) IEADJ = NE-1

C*** Find angular indices JP and JPADJ bracketing angle PSI

        IF (PSI.LE.theta(1)) THEN
            JP = 1
        ELSEIF (PSI.GE.theta(NANG)) THEN
            JP = NANG
        ELSE
            DO 20 I=2,NANG
              IF(PSI.GE.theta(I-1).AND.PSI.LT.theta(I)) THEN
                 JP = I-1
                 GOTO 21
              ENDIF
  20        CONTINUE
        ENDIF
  21    JPADJ = JP + 1
        IF (JP.EQ.NANG) JPADJ = NANG-1

C** Evaluate GCF at four surrounding tabulated grid points
        X=ZZ*RFACT
        XN=XN*RFACT
        XDL = XN*DLOG(XN)
        X15 = XN**1.5d0
c  ------- neutrons
        FIE(JP)=GCFN(NDX,IE,JP,1) + GCFN(NDX,IE,JP,2)*XN
     *          + GCFN(NDX,IE,JP,3)*XDL + GCFN(NDX,IE,JP,4)*X15
        FIEADJ(JP)=GCFN(NDX,IEADJ,JP,1) + GCFN(NDX,IEADJ,JP,2)*XN
     *          + GCFN(NDX,IEADJ,JP,3)*XDL + GCFN(NDX,IEADJ,JP,4)*X15
        FIE(JPADJ)=GCFN(NDX,IE,JPADJ,1) + GCFN(NDX,IE,JPADJ,2)*XN
     *          + GCFN(NDX,IE,JPADJ,3)*XDL + GCFN(NDX,IE,JPADJ,4)*X15
       FIEADJ(JPADJ)=GCFN(NDX,IEADJ,JPADJ,1)+GCFN(NDX,IEADJ,JPADJ,2)*XN
     *      + GCFN(NDX,IEADJ,JPADJ,3)*XDL + GCFN(NDX,IEADJ,JPADJ,4)*X15
c  ------- secondary photons
        PIE(JP)=GCFN(3,IE,JP,1) +
     *          GCFN(3,IE,JP,2)*DEXP(-X/GCFN(3,IE,JP,3))
        PIEADJ(JP)=GCFN(3,IEADJ,JP,1) +
     *          GCFN(3,IEADJ,JP,2)*DEXP(-X/GCFN(3,IEADJ,JP,3))
        PIE(JPADJ)=GCFN(3,IE,JPADJ,1) +
     *          GCFN(3,IE,JPADJ,2)*DEXP(-X/GCFN(3,IE,JPADJ,3))
        PIEADJ(JPADJ)=GCFN(3,IEADJ,JPADJ,1) +
     *          GCFN(3,IEADJ,JPADJ,2)*DEXP(-X/GCFN(3,IEADJ,JPADJ,3))

C
C** Interpolate between bracketing energies
        EFACT=(EE-EEg(IE))/(EEg(IEADJ)-EEg(IE))
        FF(JP)=FIE(JP)+(FIEADJ(JP)-FIE(JP))*EFACT
        FF(JPADJ)=FIE(JPADJ)+(FIEADJ(JPADJ)-FIE(JPADJ))*EFACT
        PF(JP)=PIE(JP)+(PIEADJ(JP)-PIE(JP))*EFACT
        PF(JPADJ)=PIE(JPADJ)+(PIEADJ(JPADJ)-PIE(JPADJ))*EFACT

C** Log-interpolate between the bracketing angles
        if (FF(JPADJ).LE.0.0D0.OR.FF(JP).LE.0.0D0) then
           print *, 'E,PSI:',EE, PSI
           print *, 'IE,IEADJ,JP,JPADJ:', IE, IEADJ, JP, JPADJ
           print *, 'ff(jp),ff(jpadj),efact:', ff(jp), ff(jpadj), efact
           print *, 'fie,fiead at jp & jpad:',
     1               fie(jp),fieadj(jp),fie(jpadj),fieadj(jpadj)
        endif

        CFN=FF(JP)+(FF(JPADJ)-FF(JP))*(PSI-theta(JP))/
     *              (theta(JPADJ)-theta(JP))
        CFP=PF(JP)+(PF(JPADJ)-PF(JP))*(PSI-theta(JP))/
     *              (theta(JPADJ)-theta(JP))

C**** Prevent large GCFs:  Secondary-photon CFP is most suseptible.
C**** This is an ad hoc fix to limit large CFP at small
C**** soure-detector distances.  Apply only to neutron GCF
C     IF (CFP.GT.1.2d0) THEN CFP=1.2d0
      IF (CFN.GT.1.2d0) THEN CFN=1.2d0

C**** Supress GCF for secondary-photons in this version of SKYCONES
      CFP = 1.d0

        RETURN
      END



      SUBROUTINE XBEAM(E,PSI,ZZ,RHOACT,RESP)
C***************************************************************************
C  Purpose:
C    This subroutine calculates the skyshine dose at a detector
C    arising from a monoenergetic and monodirectional photon beam
c    using a 3-parameter approximation for the LBRF.
C
C  Input:
C      E = Source photon energy (MeV),
C    PSI = photon emission angle (deg.)
C     ZZ = source-to-detector distance (m)
C RHOACT = air density (g cm^-3)
C
C  Output:
C     RESP = Line-beam response function (rad/photon)
C
C  Variables:
C        FK     = flux-to-dose conversion factor
C        IE     = energy group index in which energy E resides
C        IEADJ  = adjacent enrgy index
C        JP     = angular group index in which a beam angle reside
C        JPADJ  = adjacent angle index
C
C  Needed Routines:  XBINI to read LBRF approximation coefficients
C                    EAGRID Block Data to set up E and ANG grid
C
C**************************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        DIMENSION  FF(20),FIE(20),FIEADJ(20)
        COMMON /GAMDAT/ G(2,15,20,3),ANGL(2,20)
        COMMON /GAMVEC/ EE(2,15),PPSI(2,20)

        IF(E.LE.15.0) THEN
          IDAT = 1
          NE = 15
          NANG = 20
        ELSE
          IDAT = 2
          NE = 5
          NANG = 17
        END IF

        FK=1.308E-11
        RHOAIR = 0.0012D0
        RFACT = RHOACT/RHOAIR

C** Find energy indices IE and IEADJ bracketing energy E

        IF (E.LE.EE(IDAT,1)) THEN
            IE = 1
        ELSEIF (E.GE.EE(IDAT,NE)) THEN
            IE = NE
        ELSE
            DO 10 I=2,NE
              IF(E.GE.EE(IDAT,I-1).AND.E.LT.EE(IDAT,I)) THEN
                 IE = I-1
                 GOTO 11
              ENDIF
  10        CONTINUE
        ENDIF
  11    EIE = EE(IDAT,IE)
        IEADJ = IE+1
        IF(IE.EQ.NE) IEADJ = NE-1

C*** Find angular indices JP and JPADJ bracketing angle PSI

        IF (PSI.LE.PPSI(IDAT,1)) THEN
            JP = 1
        ELSEIF (PSI.GE.PPSI(IDAT,NANG)) THEN
            JP = NANG
        ELSE
            DO 20 I=2,NANG
              IF(PSI.GE.PPSI(IDAT,I-1).AND.PSI.LT.PPSI(IDAT,I)) THEN
                 JP = I-1
                 GOTO 21
              ENDIF
  20        CONTINUE
        ENDIF
  21    JPADJ = JP + 1
        IF (JP.EQ.NANG) JPADJ = NANG-1

C** Evaluate LBRF at four surrounding tabulated grid points
        X=ZZ*RFACT
        FIE(JP)=DEXP(G(IDAT,IE,JP,1) - G(IDAT,IE,JP,3)*X)
     *          *X**G(IDAT,IE,JP,2)
        FIEADJ(JP)=DEXP(G(IDAT,IEADJ,JP,1)-G(IDAT,IEADJ,JP,3)*X)
     *             *X**G(IDAT,IEADJ,JP,2)
        FIE(JPADJ)=DEXP(G(IDAT,IE,JPADJ,1)-G(IDAT,IE,JPADJ,3)*X)
     *             *X**G(IDAT,IE,JPADJ,2)
        FIEADJ(JPADJ)=DEXP(G(IDAT,IEADJ,JPADJ,1)
     *                                - G(IDAT,IEADJ,JPADJ,3)*X)
     *             *X**G(IDAT,IEADJ,JPADJ,2)
C
C** Interpolate between bracketing energies
        EFACT=(E-EE(IDAT,IE))/(EE(IDAT,IEADJ)-EE(IDAT,IE))
        FF(JP)=FIE(JP)+(FIEADJ(JP)-FIE(JP))*EFACT
        FF(JPADJ)=FIE(JPADJ)+(FIEADJ(JPADJ)-FIE(JPADJ))*EFACT

C** Log-interpolate between the bracketing angles
        FF(JP)=DLOG(FF(JP))
        FF(JPADJ)=DLOG(FF(JPADJ))
        RESP=FF(JP)+(FF(JPADJ)-FF(JP))*(PSI-PPSI(IDAT,JP))/
     *              (PPSI(IDAT,JPADJ)-PPSI(IDAT,JP))

        RESP=E*FK*(RFACT*RFACT)*DEXP(RESP)
        RETURN
      END


      SUBROUTINE NBEAM(INDX,E,PSI,ZZ,RHOACT,RESP)
C***************************************************************************
C  Purpose:
C    This subroutine calculates the skyshine dose at a detector
C    arising from a monoenergetic and monodirectional neutron beam
c    using a 3-parameter approximation for the LBRF.
C
C  Input:
C      E = Source energy (MeV),
C    PSI = neutron emission angle (deg.)
C     ZZ = source-to-detector distance (m)
C RHOACT = air density (g cm^-3)
C
C  Output:
C     RESP = Line-beam response function (Sv/neutron)
C
C  Variables:
C        FK     = flux-to-dose conversion factor
C        IE     = energy group index in which energy E resides
C        IEADJ  = adjacent enrgy index
C        JP     = angular group index in which a beam angle reside
C        JPADJ  = adjacent angle index
C
C  Needed Routines:  XBINI to read LBRF approximation coefficients
C                    EAGRID Block Data to set up E and ANG grid
C
C**************************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        DIMENSION  FF(20),FIE(20),FIEADJ(20)
        COMMON /NDAT/ GN(6,14,18,3),ANGLN(18)
        COMMON /NVEC/ EEN(14),PPSIN(18)

          NE = 14
          NANG = 18

        RHOAIR = 0.0012D0
        RFACT = RHOACT/RHOAIR

C** Find energy indices IE and IEADJ bracketing energy E

        IF (E.LE.EEN(1)) THEN
            IE = 1
        ELSEIF (E.GE.EEN(NE)) THEN
            IE = NE
        ELSE
            DO 10 I=2,NE
              IF(E.GE.EEN(I-1).AND.E.LT.EEN(I)) THEN
                 IE = I-1
                 GOTO 11
              ENDIF
  10        CONTINUE
        ENDIF
  11    EIE = EEN(IE)
        IEADJ = IE+1
        IF(IE.EQ.NE) IEADJ = NE-1

C*** Find angular indices JP and JPADJ bracketing angle PSI

        IF (PSI.LE.PPSIN(1)) THEN
            JP = 1
        ELSEIF (PSI.GE.PPSIN(NANG)) THEN
            JP = NANG
        ELSE
            DO 20 I=2,NANG
              IF(PSI.GE.PPSIN(I-1).AND.PSI.LT.PPSIN(I)) THEN
                 JP = I-1
                 GOTO 21
              ENDIF
  20        CONTINUE
        ENDIF
  21    JPADJ = JP + 1
        IF (JP.EQ.NANG) JPADJ = NANG-1

C** Evaluate LBRF at four surrounding tabulated grid points
        X=ZZ*RFACT
        FIE(JP)=DEXP(GN(INDX,IE,JP,1) + GN(INDX,IE,JP,2)*X)
     *          *X**GN(INDX,IE,JP,3)
        FIEADJ(JP)=DEXP(GN(INDX,IEADJ,JP,1)+GN(INDX,IEADJ,JP,2)*X)
     *             *X**GN(INDX,IEADJ,JP,3)
        FIE(JPADJ)=DEXP(GN(INDX,IE,JPADJ,1)+GN(INDX,IE,JPADJ,2)*X)
     *             *X**GN(INDX,IE,JPADJ,3)
        FIEADJ(JPADJ)=DEXP(GN(INDX,IEADJ,JPADJ,1)
     *                                + GN(INDX,IEADJ,JPADJ,2)*X)
     *             *X**GN(INDX,IEADJ,JPADJ,3)
C
C** Interpolate between bracketing energies
        EFACT=(E-EEN(IE))/(EEN(IEADJ)-EEN(IE))
        FF(JP)=FIE(JP)+(FIEADJ(JP)-FIE(JP))*EFACT
        FF(JPADJ)=FIE(JPADJ)+(FIEADJ(JPADJ)-FIE(JPADJ))*EFACT

C** Log-interpolate between the bracketing angles
        if (FF(JPADJ).LE.0.0D0.OR.FF(JP).LE.0.0D0) then
           print *, 'E,PSI:',E, PSI
           print *, 'IE,IEADJ,JP,JPADJ:', IE, IEADJ, JP, JPADJ
           print *, 'ff(jp),ff(jpadj),efact:', ff(jp), ff(jpadj), efact
           print *, 'fie,fiead at jp & jpad:',
     1               fie(jp),fieadj(jp),fie(jpadj),fieadj(jpadj)
        endif


        FF(JP)=DLOG(FF(JP))
        FF(JPADJ)=DLOG(FF(JPADJ))
        RESP=FF(JP)+(FF(JPADJ)-FF(JP))*(PSI-PPSIN(JP))/
     *              (PPSIN(JPADJ)-PPSIN(JP))

        RESP=E*(RFACT*RFACT)*DEXP(RESP)
        RETURN
      END


      SUBROUTINE READGS(NGAU,X,W)
C*************************************************************
C  Routine returns data needed for Gaussian quadrature
C
C  Input:
C    NGAU = Order of quadrature (4, 8, 16, or 32)
C  Return:
C      X(NGAU), W(NGAU) : Quadrature ordinates and weights
C*************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(32),W(32),GAUS(30,2)

      DATA (GAUS(I,1),I=1,2)/ .861136311594053, .339981043584856 /
      DATA (GAUS(I,2),I=1,2)/ .347854845137454, .652145154862546 /
      DATA (GAUS(I,1),I=3,6)/ .960289856497536, .796666477413627,
     1                        .525532409916329, .183434642495650/
      DATA (GAUS(I,2),I=3,6)/ .101228536290376, .222381034453374,
     1                        .313706645877887, .362683783378362 /
      DATA(GAUS(I,1),I=7,14)/ .095012509837637, .281603550779259,
     1      .458016777657227, .617876244402644, .755404408355003,
     2      .865631202387832, .944575023073233, .989400934991649/
      DATA(GAUS(I,2),I=7,14)/ .189450610455068, .182603415044924,
     1      .169156519395003, .149595988816577, .124628971255534,
     2      .095158511682493, .062253523938648, .027152459411754/
      DATA(GAUS(I,1),I=15,30)/.997263861849482, .985611511545268,
     1                        .964762255587506, .934906075937740,
     2                        .896321155766052, .849367613732570,
     3                        .794483795967942, .732182118740290,
     4                        .663044266930215, .587715757240762,
     5                        .506899908932229, .421351276130635,
     6                        .331868602282128, .239287362252137,
     7                        .144471961584796, .048307665687738/
      DATA(GAUS(I,2),I=15,30)/.007018610009470, .016274394730906,
     1                        .025392065309262, .034273862913021,
     2                        .042835898022227, .050998059262376,
     3                        .058684093478536, .065822222776362,
     4                        .072345794108849, .078193895787070,
     5                        .083311924226947, .087652093004404,
     6                        .091173878695763, .093844399080805,
     7                        .095638720079275, .096540088514728 /
C
C*** Read in quadrature ordinates and weights and use symmetry
      IF (NGAU.EQ.4) THEN
         DO 5 I = 1,2
           X(5-I) = GAUS(I,1)
5        W(5-I) = GAUS(I,2)
         DO 10 I = 1, 2
           X(I) = -X(5-I)
10       W(I) = W(5-I)
      ELSEIF (NGAU.EQ.8) THEN
         DO 20 I = 1,4
           X(9-I) = GAUS(I+2,1)
20       W(9-I) = GAUS(I+2,2)
         DO 30 I = 1 , 4
           X(I) = -X(9-I)
30         W(I) = W(9-I)
      ELSEIF(NGAU.EQ.16)THEN
         DO 40 I = 1,8
           X(8+I) = GAUS(I+6,1)
40       W(8+I) = GAUS(I+6,2)
         DO 50 I = 9, 16
           X(17-I) = -X(I)
50       W(17-I) = W(I)
      ELSE
         DO 60 I = 1,16
           X(33-I) = GAUS(I+14,1)
 60      W(33-I) = GAUS(I+14,2)
         DO 70 I = 1,16
           X(I) = -X(33-I)
70       W(I) = W(33-I)
       ENDIF
       RETURN
       END

